#!/usr/bin/perl

use strict;
use Test::More tests => 178;
use FDN::Common::Logger;
use FDN::Common::Constantes;

# 1
BEGIN { use_ok('FDN::Sql::DB_Access') }

# 2
BEGIN { use_ok('FDN::Sql::Basic') }

# 3
BEGIN { use_ok('FDN::Sql::DBC') }

# 4
BEGIN { use_ok('FDN::Sql::Workflow::Basic') }

# 5
BEGIN { use_ok('FDN::Common::Config') }

# 6
$FDN::Common::Config::CONFIG_ROOT = './t/';
my $dbh = do_connect('test');
ok ( defined $dbh, 'do_connect' );

# 7
is ( ref($dbh), 'HASH', 'do_connect' );

# 8
ok ( -f './sql/workflow.sql', 'workflow.sql' );
open FILE, '<./sql/workflow.sql';
my $workflow;
{ local $/;
  $workflow = <FILE>; }
close FILE;
my @queries = split /;\s*$/m, $workflow;
my $tmp;
my $res = TRUE;
for my $q ( @queries ) {
	next if ( $q =~ /drop table if exists SEQ/ );
	next if ( $q =~ /create table SEQ/ );
	$tmp = sql_transaction($dbh,'create',$q);
	if ( ! $tmp->{'ok'} ) {
		$res = FALSE;
		print "# Failed query: $q\n";
	}
}
# 9
ok ( $res, 'workflow.sql' );

# 10
$res = workflow_select_db('test');
is ( $res, $dbh, 'workflow_select_db' );

####
#### Task -> 57
####
my @tmp;
my $props;

# /1,8
$res = new_task('atomic','rienfair','tk48-12');
is ( $res, 1, 'new_task' );
$res = task_par_id(1);
is ( ref($res), 'HASH', 'task_par_id' );
is ( $res->{'TASK_ID'}, 1, 'task_par_id' );
is ( $res->{'TASK_TYPE'}, 'atomic', 'task_par_id' );
is ( $res->{'TASK_KEY'}, 'tk48-12', 'task_par_id' );
is ( $res->{'TASK_NAME'}, 'rienfair', 'task_par_id' );
is ( $res->{'TASK_STATE'}, 'init', 'task_par_id' );
is ( $res->{'TASK_NBCHILD'}, 0, 'task_par_id' );

# /9,15
$res = new_task('serial','2rien','plouf');
is ( $res, 2, 'new_task' );
$res = task_par_key('plouf');
is ( ref($res), 'HASH', 'task_par_key' );
is ( $res->{'TASK_ID'}, 2, 'task_par_key' );
is ( $res->{'TASK_KEY'}, 'plouf', 'task_par_key' );
is ( $res->{'TASK_TYPE'}, 'serial', 'task_par_key' );
is ( $res->{'TASK_NAME'}, '2rien', 'task_par_key' );
is ( $res->{'TASK_NBCHILD'}, 0, 'task_par_key' );

# /16,17
$res = new_task('parallel','3rien','nonplu');
is ( $res, 3, 'new_task' );
$res = task_par_id(4);
is ( $res, undef, 'task_par_id' );

# /18,21
$res = new_task('idiot','rienfair','autre');
is ( $res, 4, 'new_task' );
$res = [ task_par_name('rienfair') ];
is ( $#$res, 1, 'task_par_name' );
if ( $res->[0]->{'TASK_ID'} == 1 ) {
	$tmp[0] = $res->[0];
	$tmp[1] = $res->[1];
} else {
	$tmp[0] = $res->[1];
	$tmp[1] = $res->[0];
}
is ( $tmp[0]->{'TASK_ID'}, 1, 'task_par_name' );
is ( $tmp[1]->{'TASK_ID'}, 4, 'task_par_name' );

# /22,24
$res = [ task_par_name_not_state('rienfair','done') ];
is ( $#$res, 1, 'task_par_name_not_state' );
is ( $res->[0]->{'TASK_NAME'}, 'rienfair', 'task_par_name_not_state' );
is ( $res->[1]->{'TASK_NAME'}, 'rienfair', 'task_par_name_not_state' );

# /25
$res = [ task_par_name_state('rienfair','init') ];
is ( $#$res, 1, 'task_par_name_state' );

# /26,28
$res = task_id_par_key('nonplu');
is ( $res, 3, 'task_id_par_key' );
$res = task_id_par_key('autre');
is ( $res, 4, 'task_id_par_key' );
$res = task_id_par_key('invalid');
is ( $res, undef, 'task_id_par_key' );

# /29,38
$res = set_task_state(1,'running');
ok ( $res, 'set_task_state' );
$res = task_par_id(1);
is ( $res->{'TASK_STATE'}, 'running', 'set_task_state' );
$res = [ task_par_name_not_state('rienfair', 'running' ) ];
is ( $#$res, 0, 'set_task_state' );
$res = [ task_par_name_not_state('rienfair', 'init' ) ];
is ( $#$res, 0, 'set_task_state' );
is ( $res->[0]->{'TASK_ID'}, 1, 'set_task_state' );
$res = set_task_state(1,'failed');
ok ( $res, 'set_task_state' );
$res = task_par_id(1);
is ( $res->{'TASK_STATE'}, 'failed', 'set_task_state' );
$res = set_task_state(1,'done');
ok ( $res, 'set_task_state' );
$res = task_par_id(1);
is ( $res->{'TASK_STATE'}, 'done', 'set_task_state' );

# /39,44
$res = set_task_key(1,'clef35');
ok ( $res, 'set_task_key' );
$res = task_par_id(1);
is ( $res->{'TASK_KEY'}, 'clef35', 'set_task_key' );
$res = task_par_key('clef35');
is ( $res->{'TASK_ID'}, 1, 'set_task_key' );
$res = task_id_par_key('clef35');
is ( $res, 1, 'set_task_key' );
$res = task_par_key('tk48-12');
is ( $res, undef, 'set_task_key' );
$res = task_id_par_key('tk48-12');
is ( $res, undef, 'set_task_key' );

# /45, 46
$res = set_task_nbchild(1,8);
ok ( $res, 'set_task_nbchild' );
$res = task_par_id(1);
is ( $res->{'TASK_NBCHILD'}, 8, 'set_task_nbchild' );

# /47,52
$props = {
	'key' => 'val',
	'tab' => [ 1, 2 ]
};
set_task_properties(1,$props);
$res = get_task_properties(1);
is ( ref($res), 'HASH', 'set_task_properties' );
is ( $res->{'key'}, 'val', 'set_task_properties' );
is ( ref($res->{'tab'}), 'ARRAY', 'set_task_properties' );
ok ( eq_set( $res->{'tab'}, [ 1, 2 ] ), 'set_task_properties' );
del_task_properties(1);
$res = get_task_properties(1);
is_deeply ( $res, {}, 'get_task_properties' );
set_task_properties(1,$props);
$props->{'key'} = 'new';
update_task_properties(1,$props);
$res = get_task_properties(1);
is ( $res->{'key'}, 'new', 'update_task_properties' );

# /53
del_task(2);
$res = task_par_id(2);
is ( $res, undef, 'del_task' );
# Before 10 + (/53) -> 63

####
#### Task self -> 45
####

# /1,7
$res = new_taskself(1,8,4);
ok ( $res, 'new_taskself' );
$res = new_taskself(1,7,3);
ok ( $res, 'new_taskself' );
$res = new_taskself(1,6,2);
ok ( $res, 'new_taskself' );
$res = new_taskself(1,5,1);
ok ( $res, 'new_taskself' );
$res = new_taskself(8,9,1);
ok ( $res, 'new_taskself' );
$res = new_taskself(8,10,2);
ok ( $res, 'new_taskself' );
$res = new_taskself(9,11,1);
ok ( $res, 'new_taskself' );

# /8,20
$res = [ taskself_par_parent(1) ];
is ( $#$res, 3, 'taskself_par_parent' );
$tmp[0] = $tmp[1] = $tmp[2] = $tmp[3] = undef;
for my $t ( @$res ) {
	if ( $t->{'TASKSELF_ORDER'} == 1 ) {
		is ( $tmp[0], undef, 'taskself_par_parent' );
		$tmp[0] = $t;
	}
	if ( $t->{'TASKSELF_ORDER'} == 2 ) {
		is ( $tmp[1], undef, 'taskself_par_parent' );
		$tmp[1] = $t;
	}
	if ( $t->{'TASKSELF_ORDER'} == 3 ) {
		is ( $tmp[2], undef, 'taskself_par_parent' );
		$tmp[2] = $t;
	}
	if ( $t->{'TASKSELF_ORDER'} == 4 ) {
		is ( $tmp[3], undef, 'taskself_par_parent' );
		$tmp[3] = $t;
	}
}
is ( ref($tmp[0]), 'HASH', 'taskself_par_parent' );
is ( ref($tmp[1]), 'HASH', 'taskself_par_parent' );
is ( ref($tmp[2]), 'HASH', 'taskself_par_parent' );
is ( ref($tmp[3]), 'HASH', 'taskself_par_parent' );
is ( $tmp[0]->{'SELF_ID'}, 5, 'taskself_par_parent' );
is ( $tmp[1]->{'SELF_ID'}, 6, 'taskself_par_parent' );
is ( $tmp[2]->{'SELF_ID'}, 7, 'taskself_par_parent' );
is ( $tmp[3]->{'SELF_ID'}, 8, 'taskself_par_parent' );

# /21,23
$res = [ taskself_par_parent(8) ];
is ( $#$res, 1, 'taskself_par_parent' );
$res = [ taskself_par_parent(9) ];
is ( $#$res, 0, 'taskself_par_parent' );
$res = [ taskself_par_parent(5) ];
is ( $#$res, -1, 'taskself_par_parent' );

# /24,30
$res = taskself_par_parent_numero(1,3);
is ( ref($res), 'HASH', 'taskself_par_parent_numero' );
is ( $res->{'SELF_ID'}, 7, 'taskself_par_parent_numero' );
$res = taskself_par_parent_numero(1,5);
is ( $res, undef, 'taskself_par_parent_numero' );
$res = taskself_par_parent_numero(2,1);
is ( $res, undef, 'taskself_par_parent_numero' );
$res = taskself_par_parent_numero(8,2);
is ( $res->{'SELF_ID'}, 10, 'taskself_par_parent_numero' );
$res = taskself_par_parent_numero(9,1);
is ( $res->{'SELF_ID'}, 11, 'taskself_par_parent_numero' );
$res = taskself_par_parent_numero(9,2);
is ( $res, undef, 'taskself_par_parent_numero' );

# /31,34
$res = taskself_par_fils(7);
is ( ref($res), 'HASH', 'taskself_par_fils' );
is ( $res->{'TASK_ID'}, 1, 'taskself_par_fils' );
$res = taskself_par_fils(11);
is ( $res->{'TASK_ID'}, 9, 'taskself_par_fils' );
$res = taskself_par_fils(1);
is ( $res, undef, 'taskself_par_fils' );

# /35,37
$res = taskself_par_fils_numero(7,3);
is ( ref($res), 'HASH', 'taskself_par_fils_numero' );
is ( $res->{'TASK_ID'}, 1, 'taskself_par_fils_numero' );
$res = taskself_par_fils_numero(7,2);
is ( $res, undef, 'taskself_par_fils_numero' );

# /38,44
$res = taskself_par_parent_fils(1,7);
is ( ref($res), 'HASH', 'taskself_par_parent_fils' );
is ( $res->{'TASK_ID'}, 1, 'taskself_par_parent_fils' );
is ( $res->{'SELF_ID'}, 7, 'taskself_par_parent_fils' );
is ( $res->{'TASKSELF_ORDER'}, 3, 'taskself_par_parent_fils' );
$res = taskself_par_parent_fils(1,9);
is ( $res, undef, 'taskself_par_parent_fils' );
$res = taskself_par_parent_fils(9,11);
is ( $res->{'TASK_ID'}, 9, 'taskself_par_parent_fils' );
is ( $res->{'SELF_ID'}, 11, 'taskself_par_parent_fils' );

# /45,
del_taskself(9,11);
$res = taskself_par_parent_fils(9,11);
is ( $res, undef, 'del_taskself' );
# Before 63 + (/45) -> 108

####
#### Events -> 25
####

# /1,5
$res = new_event('ev1');
is ( $res, 1, 'new_event' );
$res = event_par_id(1);
is ( ref($res), 'HASH', 'event_par_id' );
is ( $res->{'EVENT_ID'}, 1, 'event_par_id' );
is ( $res->{'EVENT_EVENT'}, 'ev1', 'event_par_id' );
$res = event_par_id(2);
is ( $res, undef, 'event_par_id' );

# /6,12
$res = new_event('ev2');
is ( $res, 2, 'new_event' );
$res = event_par_event('ev2');
is ( ref($res), 'HASH', 'event_par_event' );
is ( $res->{'EVENT_ID'}, 2, 'event_par_event' );
is ( $res->{'EVENT_EVENT'}, 'ev2', 'event_par_event' );
$res = event_par_event('ev1');
is ( $res->{'EVENT_ID'}, 1, 'event_par_event' );
is ( $res->{'EVENT_EVENT'}, 'ev1', 'event_par_event' );
$res = event_par_event('ev3');
is ( $res, undef, 'event_par_event' );

# /13,17
$props = {
	'a' => 1,
	'b' => 2
};
set_event_properties(1,$props);
$res = get_event_properties(1);
is ( ref($res), 'HASH', 'set_event_properties' );
is_deeply ( $res, $props, 'get_event_properties' );
$props->{'a'} = 4;
$props->{'key'} = 'value';
update_event_properties(1,$props);
$res = get_event_properties(1);
is_deeply ( $res, $props, 'get_event_properties' );
del_event_properties(1);
$res = get_event_properties(1);
is_deeply ( $res, {}, 'del_event_properties' );
update_event_properties(1,$props);
$res = get_event_properties(1);
is_deeply ( $res, $props, 'update_event_properties' );

# /18
$res = lock_event();
ok ( $res, 'lock_event' );

# /19
$res = unlock_event();
ok ( $res, 'unlock_event' );
$res = task_par_id(1);
is ( ref($res), 'HASH', 'unlock_event' );

# /20
del_event(3);
$res = event_par_id(3);
is ( $res, undef, 'del_event' );

# /21
del_event_par_event('ev2');
$res = event_par_id(2);
is ( $res, undef, 'del_event_par_event' );

# Before 108 + (/21) ->  129

####
#### Waiting -> 31
####

# /1,6
$res = new_waiting(5,'ev4');
is ( $res, 1, 'new_waiting' );
$res = waiting_par_id(1);
is ( ref($res), 'HASH', 'waiting_par_id' );
is ( $res->{'WAITING_ID'}, 1, 'waiting_par_id' );
is ( $res->{'TASK_ID'}, 5, 'waiting_par_id' );
is ( $res->{'WAITING_EVENT'}, 'ev4', 'waiting_par_id' );
$res = waiting_par_id(2);
is ( $res, undef, 'waiting_par_id' );

# /7
$res = new_waiting(6,'ev4');
is ( $res, 2, 'new_waiting' );

# /8
$res = new_waiting(5,'ev5');
is ( $res, 3, 'new_waiting' );

# /9,10
$res = [ waiting_par_event('ev5') ];
is ( $#$res, 0, 'waiting_par_event' );
is ( $res->[0]->{'WAITING_ID'}, 3, 'waiting_par_event' );

# /11,13
$res = [ waiting_par_event('ev4') ];
is ( $#$res, 1, 'waiting_par_event' );
is ( $res->[0]->{'WAITING_EVENT'}, 'ev4', 'waiting_par_event' );
is ( $res->[1]->{'WAITING_EVENT'}, 'ev4', 'waiting_par_event' );

# /14
$res = [ waiting_par_event('ev2') ];
is ( $#$res, -1, 'wainting_par_event' );

# /15,18
$res = [ waiting_par_task(6) ];
is ( $#$res, 0, 'waiting_par_task' );
is ( $res->[0]->{'TASK_ID'}, 6, 'waiting_par_task' );
is ( $res->[0]->{'WAITING_ID'}, 2, 'waiting_par_task' );
is ( $res->[0]->{'WAITING_EVENT'}, 'ev4', 'waiting_par_task' );

# /19,21
$res = [ waiting_par_task(5) ];
is ( $#$res, 1, 'waiting_par_task' );
is ( $res->[0]->{'TASK_ID'}, 5, 'waiting_par_task' );
is ( $res->[1]->{'TASK_ID'}, 5, 'waiting_par_task' );

# /22,25
$res = [ tous_waiting() ];
is ( $#$res, 2, 'tous_waiting' );
ok ( $res->[0]->{'WAITING_ID'}, 'tous_waiting' );
ok ( $res->[1]->{'WAITING_ID'}, 'tous_waiting' );
ok ( $res->[2]->{'WAITING_ID'}, 'tous_waiting' );

# /26
del_waiting(3);
$res = waiting_par_id(3);
is ($res, undef, 'del_waiting' );

# Before 129 + (/26) -> 155

####
#### Run -> 26
####

# /1,7
$res = new_run(2,'ev12');
is ( $res, 1, 'new_run' );
$res = run_par_id(1);
is ( ref($res), 'HASH', 'run_par_id' );
is ( $res->{'RUN_ID'}, 1, 'run_par_id' );
is ( $res->{'TASK_ID'}, 2, 'run_par_id' );
is ( $res->{'RUN_EVENT'}, 'ev12', 'run_par_id' );
is ( $res->{'RUN_STATE'}, 'free', 'run_par_id' );
$res = run_par_id(2);
is ( $res, undef, 'run_par_id' );

# /8
$res = new_run(2,'init');
is ( $res, 2, 'new_run' );

# /9
$res = new_run(3,'init');
is ( $res, 3, 'new_run' );

# /10,11
$res = [ run_par_task(3) ];
is ( $#$res, 0, 'run_par_task' );
is ( $res->[0]->{'RUN_ID'}, 3, 'run_par_task' );

# /12
$res = [ run_par_task(1) ];
is ( $#$res, -1, 'run_par_task' );

# /13,15
$res = [ run_par_task(2) ];
is ( $#$res, 1, 'run_par_task' );
is ( $res->[0]->{'TASK_ID'}, 2, 'run_par_task' );
is ( $res->[1]->{'TASK_ID'}, 2, 'run_par_task' );

# /16
$res = [ run_par_state('free') ];
is ( $#$res, 2, 'run_par_state' );

# /17
$res = [ run_par_state('busy') ];
is ( $#$res, -1, 'run_par_state' );

# /18,21
$res = set_run_state(1,'busy');
ok ( $res, 'set_run_state' );
$res = run_par_id(1);
is ( $res->{'RUN_STATE'}, 'busy', 'set_run_state' );
$res = [ run_par_state('free') ];
is ( $#$res, 1, 'set_run_state' );
$res = [ run_par_state('busy') ];
is ( $#$res, 0, 'set_run_state' );

# /22
$res = lock_run();
ok ( $res, 'lock_run' );

# /23
$res = unlock_run();
ok ( $res, 'unlock_run' );
# Before 155 + (/23) -> 178




